# frozen_string_literal: true

require 'cgi'

module Jekyll
  class Embed
    class Content
      URL_RE = /(<p[^>]*>[\s\n]*(<a[^>]*>)?[\s\n]*(https?:\/\/[^<\s\n]+)[\s\n]*(<\/a>)?[\s\n]*<\/p>)/m.freeze

      class << self
        # Find URLs on paragraphs.  We do it after rendering because
        # sometimes we use HTML instead of pure Markdown and this way we
        # catch both.
        def embed!(content)
          content.gsub!(URL_RE) do |_|
            Jekyll::Embed.embed CGI.unescapeHTML($3)
          end
        end
      end
    end
  end
end

Jekyll::Hooks.register :posts, :post_convert do |post|
  Jekyll::Embed::Content.embed! post.content
end
